MinimalArchaeology = {}

MinimalArchaeology['skill'] = {}	 -- Store information about the players skill
MinimalArchaeology['artifacts'] = {} -- Store information about current artifacts
MinimalArchaeology['barlinks'] = {1,2,3,4,5,6,7,8,9,10} -- Store information about bar links
MinimalArchaeology['ArtifactBars'] = {} -- References for the artifact bars

MinimalArchaeologyOptions = {} -- Store options

SLASH_MINIMALARCHAEOLOGY1 = "/minarch"
SlashCmdList["MINIMALARCHAEOLOGY"] = function(msg, editBox)
    MinimalArchaeologyFrame:Show();
end

function MinimalArchaeology:OnLoad(self)
	-- Register Events
	MinimalArchaeologyFrame:RegisterForDrag("LeftButton")
	
	--MinimalArchaeologyFrame:RegisterEvent("ARTIFACT_UPDATE"); -- Will need this later for keystones
	
	MinimalArchaeologyFrame:RegisterEvent("ARTIFACT_HISTORY_READY");
	MinimalArchaeologyFrame:RegisterEvent("ARTIFACT_COMPLETE");
	MinimalArchaeologyFrame:RegisterEvent("ARTIFACT_DIG_SITE_UPDATED");
	MinimalArchaeologyFrame:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
	MinimalArchaeologyFrame:RegisterEvent("SKILL_LINES_CHANGED");
	
	MinimalArchaeologyFrame:RegisterEvent("BAG_UPDATE");	-- Might need this later
	
	MinimalArchaeologyFrame:RegisterEvent("ADDON_LOADED");  -- Apply SavedVariables
	
	for i=1,10 do -- Artifact Arrays
		MinimalArchaeology['artifacts'][i] = {};
	end
	
	-- Set Frames
	MinimalArchaeology['ArtifactBars'][1] = MinimalArchaeologyFrameStatusBar1;
	MinimalArchaeology['ArtifactBars'][2] = MinimalArchaeologyFrameStatusBar2;
	MinimalArchaeology['ArtifactBars'][3] = MinimalArchaeologyFrameStatusBar3;
	MinimalArchaeology['ArtifactBars'][4] = MinimalArchaeologyFrameStatusBar4;
	MinimalArchaeology['ArtifactBars'][5] = MinimalArchaeologyFrameStatusBar5;
	MinimalArchaeology['ArtifactBars'][6] = MinimalArchaeologyFrameStatusBar6;
	MinimalArchaeology['ArtifactBars'][7] = MinimalArchaeologyFrameStatusBar7;
	MinimalArchaeology['ArtifactBars'][8] = MinimalArchaeologyFrameStatusBar8;
	MinimalArchaeology['ArtifactBars'][9] = MinimalArchaeologyFrameStatusBar9;
	MinimalArchaeology['ArtifactBars'][10] = MinimalArchaeologyFrameStatusBar10;
	
	MinimalArchaeology:UpdateMinArchRankBar()
	
	ChatFrame1:AddMessage("Minimal Archaeology Loaded!");
end

function MinimalArchaeology:OnEvent(event, ...)
	if (event == "ADDON_LOADED" and MinimalArchaeology ~= nil) then
		if (MinimalArchaeologyOptions['MinimapPos'] ~= nil) then
			MinimalArchaeology:MinimapButtonReposition();
		else
			MinimalArchaeologyOptions['MinimapPos'] = 45;
		end
	end

	MinimalArchaeology:UpdateMinArchRankBar();
end

function MinimalArchaeology:UpdateMinArchSkill()	-- Update archaeology skill information
	local _, _, arch = GetProfessions();
	if arch then
		local name, _, rank, maxRank = GetProfessionInfo(arch);
		MinimalArchaeology['skill']['name'] = name;
		MinimalArchaeology['skill']['rank'] = rank;
		MinimalArchaeology['skill']['maxRank'] = maxRank;
	else
		MinimalArchaeology['skill']['name'] = "Archaeology";
		MinimalArchaeology['skill']['rank'] = 0;
		MinimalArchaeology['skill']['maxRank'] = 0;
	end
end

function MinimalArchaeology:UpdateMinArchSkillBar(SkillBar)	-- Update a status bar to represent the skill information
	local skill = MinimalArchaeology['skill'];
	
	SkillBar:SetMinMaxValues(0, skill['maxRank']);
	SkillBar:SetValue(skill['rank']);
	SkillBar.text:SetText(skill['name'].." "..skill['rank'].."/"..skill['maxRank']);
end

function MinimalArchaeology:UpdateMinArchArtifact(ArtifactIndex)	-- Update artifact details, only keep relevant data
	local numArtifacts = GetNumArtifactsByRace(ArtifactIndex);
	local RaceName, _, _, RaceitemID = GetArchaeologyRaceInfo(ArtifactIndex);
	
	MinimalArchaeology['artifacts'][ArtifactIndex]['race'] = RaceName;
	
	if (numArtifacts == 0) then	-- The Artifact hasn't been discovered
		MinimalArchaeology['artifacts'][ArtifactIndex]['numKeystones'] = 0;
		MinimalArchaeology['artifacts'][ArtifactIndex]['progress'] = 0;
		MinimalArchaeology['artifacts'][ArtifactIndex]['modifier'] = 0;
		MinimalArchaeology['artifacts'][ArtifactIndex]['total'] = 0;
		MinimalArchaeology['artifacts'][ArtifactIndex]['canSolve'] = false;
	else
		SetSelectedArtifact(ArtifactIndex);
		local Name, Description, Rarity, Icon, SpellDescription, NumKeystones = GetSelectedArtifactInfo();
		local Progress, Modifier, Total = GetArtifactProgress();
		
		MinimalArchaeology['artifacts'][ArtifactIndex]['numKeystones'] = NumKeystones;
		MinimalArchaeology['artifacts'][ArtifactIndex]['progress'] = Progress;
		MinimalArchaeology['artifacts'][ArtifactIndex]['modifier'] = Modifier;
		MinimalArchaeology['artifacts'][ArtifactIndex]['total'] = Total;
		MinimalArchaeology['artifacts'][ArtifactIndex]['canSolve'] = CanSolveArtifact();
		MinimalArchaeology['artifacts'][ArtifactIndex]['name'] = Name;
		MinimalArchaeology['artifacts'][ArtifactIndex]['rarity'] = Rarity;
		MinimalArchaeology['artifacts'][ArtifactIndex]['description'] = SpellDescription;
	end	
	
	
end

function MinimalArchaeology:UpdateMinArchArtifactBar(ArtifactIndex, ArtifactBar)
	local artifact = MinimalArchaeology['artifacts'][ArtifactIndex];
	local keyStars = "";
	
	ArtifactBar:SetMinMaxValues(0, artifact['total']);
	ArtifactBar:SetValue(min(artifact['progress']+artifact['modifier'], artifact['total']));
		
	if (artifact['numKeystones'] > 0 and artifact['total'] > 0) then
		
		for i=1,artifact['numKeystones'] do
			keyStars = "*"..keyStars;
		end
	end
	
	if (artifact['modifier'] > 0) then
		ArtifactBar.text:SetText(keyStars..artifact['race'].." "..(artifact['progress']+artifact['modifier']).." (+"..artifact['modifier']..")/"..artifact['total']);
	else
		ArtifactBar.text:SetText(keyStars..artifact['race'].." "..artifact['progress'].."/"..artifact['total']);
	end
	
	if (artifact['canSolve']) then
		PlaySoundFile("Sound\\interface\\MapPing.wav")
		ArtifactBar.solveButton:Enable();
	else
		ArtifactBar.solveButton:Disable();
	end	
end

function MinimalArchaeology:MinArchSolve(BarIndex)
	SetSelectedArtifact(MinimalArchaeology['barlinks'][BarIndex]);
	SolveArtifact();
end

function MinimalArchaeology:UpdateMinArchRankBar()
	MinimalArchaeology:UpdateMinArchSkill();
	MinimalArchaeology:UpdateMinArchSkillBar(MinimalArchaeologyFrameSkillBar);
	
	local ActiveBarIndex = 0;
	
	for i=1,10 do
		MinimalArchaeology:UpdateMinArchArtifact(i); -- Update each artifact
		
		local artifact = MinimalArchaeology['artifacts'][i];
		if (artifact['total'] > 0) then
			ActiveBarIndex = ActiveBarIndex + 1;
			MinimalArchaeology:UpdateMinArchArtifactBar(i,MinimalArchaeology['ArtifactBars'][ActiveBarIndex]);
			MinimalArchaeology['ArtifactBars'][ActiveBarIndex]:Show();
			MinimalArchaeology['barlinks'][ActiveBarIndex] = i;
		end
	end
	
	local MinArchFrameHeight = 310;
	
	for i=ActiveBarIndex+1, 10 do
		MinimalArchaeology['ArtifactBars'][i]:Hide();
		MinArchFrameHeight = MinArchFrameHeight - 25;
	end
	
	MinimalArchaeologyFrame:SetHeight(MinArchFrameHeight);
	
end

function MinimalArchaeology:ArtifactToolTip(BarIndex)
	local artifact = MinimalArchaeology['artifacts'][MinimalArchaeology['barlinks'][BarIndex]];
	GameTooltip:SetOwner(MinimalArchaeology['ArtifactBars'][BarIndex], "ANCHOR_BOTTOMRIGHT");
	if (artifact['rarity'] == 1) then
		GameTooltip:AddLine(artifact['name'], 0, 0.4, 0.8, 1);
	else
		GameTooltip:AddLine(artifact['name'], GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b, 1);
	end
	GameTooltip:AddLine(artifact['description'], NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
	GameTooltip:Show();
end


-- Minimap Functions

function MinimalArchaeology:MinimapButtonReposition()
	MinimalArchaeologyMinimapButton:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-(80*cos(MinimalArchaeologyOptions.MinimapPos)),(80*sin(MinimalArchaeologyOptions.MinimapPos))-52)
end

-- Only while the button is dragged this is called every frame
function MinimalArchaeology:MinimapButtonDraggingFrameOnUpdate()

	local xpos,ypos = GetCursorPosition()
	local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom()

	xpos = xmin-xpos/UIParent:GetScale()+70 -- get coordinates as differences from the center of the minimap
	ypos = ypos/UIParent:GetScale()-ymin-70

	MinimalArchaeologyOptions.MinimapPos = math.deg(math.atan2(ypos,xpos)) -- save the degrees we are relative to the minimap center
	MinimalArchaeology:MinimapButtonReposition() -- move the button
end

-- Put your code that you want on a minimap button click here.
function MinimalArchaeology:MinimapButtonOnClick()
	if (MinimalArchaeologyFrame:IsVisible()) then
		MinimalArchaeologyFrame:Hide();
	else
		MinimalArchaeologyFrame:Show();
	end
end